::mercModelPath <- "models/player/mercenary.mdl";
::heldWeapons <- player_collection(null);

PrecacheModel(mercModelPath);
PrecacheScriptSound("CaptureFlag.FlagSpawn")

PrecacheSound("vo/mercenary_paincrticialdeath01.wav")
PrecacheSound("vo/mercenary_paincrticialdeath02.wav")
PrecacheSound("vo/mercenary_paincrticialdeath03.wav")
PrecacheSound("vo/mercenary_paincrticialdeath04.wav")

vo_death_sounds <- ["vo/mercenary_paincrticialdeath01.wav", "vo/mercenary_paincrticialdeath02.wav", "vo/mercenary_paincrticialdeath03.wav", "vo/mercenary_paincrticialdeath04.wav"]

OnGameEvent("player_spawn", 0, function(player, params)
{
    player.SetScriptOverlayMaterial("");

    if (redTeamOnly && player.GetTeam() != TF_TEAM_RED)
    {
        player.SwitchTeam(TF_TEAM_RED);
        player.ForceRespawn();
    }
});

OnGameEventPost("player_spawn", 0, function (player, params)
{
    local weaponTT = GetWeaponTypeByName(GUNGAME_ORDER[currentWeapons[player.entindex()]]);
    local playerClass = "playerclass" in weaponTT ? weaponTT.playerclass : TF_CLASS_SOLDIER;
    player.SetPlayerClass(playerClass);
    player.Regenerate(true);
    SetPropInt(player, "m_bForcedSkin", 1);
    SetPropInt(player, "m_nForcedSkin", 5);

    player.SetCustomModelWithClassAnimations(mercModelPath);
    player.SetModelScale(1.0, 0);

    local itemsToKill = [];
    for (local item = player.FirstMoveChild(); item != null; item = item.NextMovePeer())
        if (startswith(item.GetClassname(), "tf_we"))
            itemsToKill.push(item);
    foreach (item in itemsToKill)
        item.Kill();

    //player.AddCustomAttribute("move speed bonus", 1.5, -1);
    player.AddCustomAttribute("cancel falling damage", 1, -1);
    player.AddCustomAttribute("crit mod disabled hidden", 0, -1);
    player.AddCustomAttribute("voice pitch scale", 0, -1);

    player.SetHealth(150);
    player.SetMaxHealth(150);
    player.RemoveCustomAttribute("max health additive penalty");
    player.AddCustomAttribute("max health additive penalty", -50, -1);

    local steamId = GetPropString(player, "m_szNetworkIDString");
    local indexStr = steamId.len() > 5 ? steamId.slice(5) : "4444444";
    local index = indexStr.tointeger();
    SetEntityColor(player, [index % 256, (index >> 8) % 256, (index >> 16) % 256]);

    PullOutWeapon(player, GetWeaponTypeByName("crowbar"));
    PullOutWeapon(player, GetCurrentGunGameWeapon(player));

    player.AddCondEx(TF_COND_INVULNERABLE, GetRoundState() == 3 ? 5 : 3, null);

    RunWithDelayPlayer(this, -1, function(player)
    {
        player.SetHealth(150);
        player.SetMaxHealth(150);
    }, player);

    RunWithDelayPlayer(this, GetRoundState() == 3 ? 5 : 3, function(player)
    {
        SetPropInt(player, "m_nForcedSkin", 4);
    }, player)

    if (GetRoundState() >= 4)
        CreateAoE(player.GetOrigin(), 500, function(target, deltaVector, distance) {
            for (local i = 0; i < 10; i++)
            EmitSoundEx({
                sound_name = "CaptureFlag.FlagSpawn",
                volume = 1,
                soundlevel = 150,
                flags = 1,
                channel = i,
                entity = player,
                speaker_entity = player
            });
        }, function(target, deltaVector, distance) {})
});

OnGameEvent("player_death", 0, function(player, params)
{
    player.EmitSound(vo_death_sounds[RandomInt(0, vo_death_sounds.len() - 1)]);
    RunWithDelayPlayer(this, 3, function(player)
    {
        player.ForceRespawn();
    }, player);
});

OnGameEvent("player_death_post", 0, function(player, params)
{
    local ragdoll = GetPropEntity(player, "m_hRagdoll");
    SetPropBool(ragdoll, "m_bGib", false);
    SetPropInt(ragdoll, "m_clrRender", 4444444);
});

AddTimer(-1, 0, function ()
{
    foreach (player in GetAlivePlayers())
    {
        if (GetPropInt(player, "m_nButtons") & IN_ATTACK)
        {
            player.RemoveCond(TF_COND_INVULNERABLE);
            SetPropInt(player, "m_nForcedSkin", 4);
        }
    }
})

::SetEntityColor <- function(entity, rgb)
{
    local color = rgb[0] | (rgb[1] << 8) | (rgb[2] << 16);
    SetPropInt(entity, "m_clrRender", color);
}